!  Routine to prepare pressure head & relative saturation (chi) data for export to ijz & .3d formats
!  Calling routine should set value of dcf if it is not initialized in main.
!  Rex L. Baum, 25 Feb 2010, latest revision 20 Jan 2011 (improved commenting)
	subroutine pchijz(i,jf,delh,newdep,ulog)
	use grids; use input_vars 
	use model_vars
	use input_file_defs
	implicit none
!	integer,parameter:: double=kind(1d0)
	integer::i,jf,mnp(1),ulog,mxp(1),m,mext(2),mctr,mm
	real (double)::delh,newdep,pab(nzs+1),zns,zinc,pmn
	real (double):: dwat,z,ptop,pbot,zwat,zbot,ztop,chtop,chbot
	real (double):: pmx,zord(2) ! added zord() 1/3/2011 RLB
	logical :: pmask(nzs+1)
! compute water-table elevations from surface elevation and depths
	zbot=elev(i)-zmax(i)
	ztop=elev(i)-zmin
	zord=0.
	zns=float(nzs) ! moved out of loop to correct calculation of zord() 1/6/2011
	zinc=(zmax(i)-zmin)/zns
!
	ptop=p(1)
	pbot=p(nzs+1)
	chtop=chi(1)
	chbot=chi(nzs+1)
! estimate height of water table from results of unsaturated infiltration model
	if(dcf>0. .and. unsat(zo(i))) then
	  if(rikzero(i)>=0.0) then
	    zwat=elev(i)-(dusz-delh)
	  else
	    zwat=elev(i)-newdep
	  end if
	else if (dcf<=0.) then
! estimate height of water table from results of saturated infiltration model
	  chi=1.d0
	  pab=abs(p)
	  mnp=minloc(pab)
	  pmn=minval(pab)
	  z=zmin+zinc*float(mnp(1)-1)
	  if(pmn>1.e-03 .and. mnp(1)>1) then
	    dwat=z-zinc+2*zinc*(0.d0-p(mnp(1)-1))/(p(mnp(1)+1)-p(mnp(1)-1))
	  else if(pmn<=1.e-03) then
	    dwat=z
	  end if
	  if(rikzero(i)>=0.0) then
	    zwat=elev(i)-dwat
	  else
	    zwat=elev(i)-newdep
	  end if
	end if
! find maximum and minimum above water table
	pmask=.false.
	z=ztop
	do m=2,nzs
	  z=z-zinc
	  if(z>zwat) then
	    pmask(m)=.true.
	  end if
	end do
	mnp=minloc(p,pmask)
	pmn=minval(p,1,pmask)
	mxp=maxloc(p,pmask)
	pmx=maxval(p,1,pmask)
	mext=0; mctr=0 ! added 12/28/2010 to correct output errors
	if(mxp(1)>1) then ! order maximum and minimum values above water table
	  mext(1)=mxp(1)
	  mctr=1
	  if(pmn<0.) then
	    mctr=2
	    if(mnp(1)>mxp(1)) then
	      mext(2)=mnp(1)
	    else
	      mext(1)=mnp(1)
	      mext(2)=mxp(1)          
	    end if
	  end if
	else if(mnp(1)>0 .and. pmn>0.) then
	  mext(1)=mnp(1)
	  mctr=1
	else
	  mctr=0
	end if
	if(mctr>0) then
	  do m=1,mctr
	    mm=mext(m)-1
	    zord(m)=ztop-zinc*float(mm)
!	    zord(m)=ztop-float(mext(m)-1)*zinc
!	    write(*,*) 'ix(i),jy(i),m,mext,ztop,zord(m)', ix(i),jy(i),m,mext(m),ztop,zord(m)&
!	    &,pmn,mnp(1)
	  end do
	end if
	if(flag==-5) then
! Save data in ijz format
	  if(unsat0) then
  	    write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	    & ix(i),jy(i),ztop,ptop,chtop 
  	    if(mctr>0) then
	      do m=1,mctr ! added 12/28/2010 to correct output errors
  	        write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	        & ix(i),jy(i),zord(m),p(mext(m)),chi(mext(m)) 
	      end do
	    end if
	    if(zwat < ztop .and. ptop < 0. .and. zwat > zbot) then ! eliminate repeated entries for water table
  	      write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	      & ix(i),jy(i),zwat,0.d0,1.d0 
  	    end if
  	    write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	    & ix(i),jy(i),zbot,pbot,chbot 
	  else
  	    write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	    & ix(i),jy(i),ztop,ptop 
  	    if(mctr>0) then
	      do m=1,mctr
  	        write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	        & ix(i),jy(i),zord(m),p(mext(m))
  	      end do
  	    end if
	    if(zwat < ztop .and. ptop < 0. .and. zwat > zbot) then ! eliminate repeated entries for water table
  	      write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	      & ix(i),jy(i),zwat,0.d0 
  	    end if
  	    write(uijz(jf),fmt='(2(i6,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	    & ix(i),jy(i),zbot,pbot
	  end if
! save data in xyz format (updated to allow only one column for VisIt Point3D format, 12/27/2010)
	else if(flag==-6)then
!	  if(unsat0) then
!  	    write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
!  	    & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
!	    & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),ztop,ptop,chtop 
!  	    if(mctr>0) then
!	      do m=1,mctr
! 	        write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
!  	        & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
! 	        & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),zord(m),p(mext(m)),chi(mext(m))
!	      end do
!	    end if
!	    if(zwat < ztop .and. ptop < 0. .and. zwat > zbot) then ! eliminate repeated entries for water table
! 	      write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
! 	      & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
! 	      & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),zwat,0.d0,1.d0 
! 	    end if
! 	    write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
! 	    & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
! 	    &(yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),zbot,pbot,chbot 
!	  else
  	    write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	    & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
  	    & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),ztop,ptop 
  	    if(mctr>0) then
	      do m=1,mctr
  	        write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	        & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
  	        & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),zord(m),p(mext(m))
	      end do
	    end if
	    if(zwat < ztop .and. ptop < 0. .and. zwat > zbot) then ! eliminate repeated entries for water table
  	      write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	      & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
  	      & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),zwat,0.d0 
  	    end if
  	    write(uijz(jf),fmt='(2(g18.9,1x),g15.5,1x,2(g11.4,1x))',err=11)&
  	    & (xllc+(0.5d0+dfloat(ix(i)-1))*celsiz),&
  	    & (yllc+(0.5d0+dfloat(jy(i)-1))*celsiz),zbot,pbot
!	  end if
	end if
!	pause 'Press return to continue'
	return
  11	continue
  	write(*,*) 'Error writing ijz output file, at step ',jf
  	write(ulog,*) 'Error writing ijz output file, at step ',jf
  	close (ulog)
  	stop '11 in subroutine pchijz()'
	end subroutine pchijz