!  Program to compute pore-pressure response and 
!  factor of safety for saturated and unsaturated infiltration.
!  by Rex L. Baum and W.Z. Savage, USGS
!
!
!  ==============================================================
!  TRIGRS-P :: the robabilistic version of TRIGRS. It treats the  
!  mechanical and hydrological parameters as randomly distributed
!  variables (normally or uniformly). TRIGRS-P has TRIGRS as a 
!  deterministic limit.
!  By S. Raia, CNR-IRPI, 2011
!   
	program trigrs
	use input_file_defs; use input_vars
	use grids; use model_vars
	implicit none
	integer, parameter::ulen=25 !Added 12/7/2010 RLB
 	integer:: grd
	integer:: i,j,k,imx1,m,mnd
  	integer:: nodata,sctr, umax !added umax, 12/6/2010, RLB
	integer:: ncol,nrow,u(ulen),maxzo,ncc,nccs
	real               :: x1
	character (len=1)  :: tb
	character (len=255):: outfil 
  	character (len=14):: fminfil='TRfs_min_'
  	character (len=14):: zfminfil='TRz_at_fs_min_'
  	character (len=14):: pminfil='TRp_at_fs_min_'
  	character (len=18):: profil='TRlist_z_p_fs_'
	character (len=14):: header(6)
  	character (len=13):: ncvfil='TRnon_convrg_'  	
	character (len=8):: date
	character (len=10):: time
	character (len=4):: stp !,grxt ! Added grxt 4/14/2010 moved to module "grids" 4/26/10
	character (len=31):: scratch,irfil
	character (len=7):: vrsn
	character (len=11):: bldate
	logical :: lwarn
! SR__
        integer:: inorm , seed1, seed2, igen , count
        character (len= 5) :: zone
        integer            :: values (8)
        
        
! __SR

! SR__tmp

! 
         character (len=25):: fminfilz5_1='fs1_min_'
         character (len=25):: fminfilz5_2='fs2_min_'
        integer ::  totcells_z5=0, totfsmin_z5=0
        real , allocatable :: z5fs_1(:) ,  z5fs_2(:)
        real :: ratio 
        real, allocatable :: single_fs(:)
! tmp__SR


! first executable statement ............	

! first executable statement ............	
     	call date_and_time(date,time,zone,values)
	test=-9999.D0; test1=-9999.
	u=(/11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,&
     &	28,29,30,31,32,33,34,35/)      ! unit 12 --> stochastic_data 
	pi=3.141592653589793
	dg2rad=pi/180.D0
	vrsn='2.0.08n'; bldate='03 Feb 2011'
	write (*,*) ''
	write (*,*) 'TRIGRS: Transient Rainfall Infiltration'
	write (*,*) 'and Grid-based Regional Slope-Stability'
	write (*,*) '               Analysis'
	write (*,*) '       Version ', vrsn,', ',bldate
	write (*,*) '  By Rex L. Baum and William Z. Savage'
	write (*,*) '       U.S. Geological Survey'
	write (*,*) '-----------------------------------------'
	write (*,*) 'Portions of this program include material'
	write (*,*) '           copyrighted (C) by'
	write (*,*) '      Absoft Corporation 1988-2010.'
	write (*,*) ''
 	tb=char(9)
!  open log file
	outfil='TrigrsLog.txt'
	open (u(19),file=trim(outfil),status='unknown',err=410)
	write (u(19),*) ''
     	write (u(19),*) 'Starting TRIGRS ', vrsn,' ',bldate
     	write (u(19),*) 'Date ',date(5:6),'/',date(7:8),'/',date(1:4)
	write (u(19),*) 'Time ',time(1:2),':',time(3:4),':',time(5:6)
!  read initialization file
	call trini(u(19),u(7),dg2rad)
! Allocate & initialize arrays needed for runoff routing
	grd=row*col
	imx1=imax
	allocate (pf2(grd),indx(imax),nxt(imax))
!	allocate (dsctr(imax+1),dsc(nwf),slo(imax)) !2/2/2011 moved dsc() to rnoff, RLB
	allocate (dsctr(imax+1),slo(imax))
	allocate (pf1(grd),rizero(imax))
	allocate (ri(imax),rik(imax*nper),ro(imax))
!	allocate (wf(nwf),rikzero(imax),temp(col)) !2/2/2011 moved dsc() to rnoff, RLB
	allocate (rikzero(imax),temp(col))
	allocate (depth(imax),zmax(imax))
	allocate (zo(imax),ir(imax),tfg(imax))

        allocate (z5fs_1(imax*nout), z5fs_2(imax*nout))
        allocate (single_fs(nout))
!
! SR__ 
! dont allocate if, for the given parameter, all sd (standard deviation are zero) 
        if (c_sd_glob_flag  ) allocate (c_norm  (imax))
        if (phi_sd_glob_flag) allocate (phi_norm(imax))
        if (uws_sd_glob_flag) allocate (uws_norm(imax))
        if (dif_sd_glob_flag) allocate (dif_norm(imax))
        if (ks_sd_glob_flag ) allocate (ks_norm (imax))
        if (ths_sd_glob_flag) allocate (ths_norm(imax))
        if (thr_sd_glob_flag) allocate (thr_norm(imax))
        if (alp_sd_glob_flag) allocate (alp_norm(imax))
! __SR
!
	if(flag<= -4) then !flag=-4, -5, -6 produces ijz or xyz output, 12/22/2010 RLB
	  allocate (elev(imax)) ! added 4/21/2010
	  elev=0.
	end if
	 pf2=0
	 indx=0
	 nxt=0
	 dsctr=0
!	 dsc=0
	 zo=0
	 pf1=0.
	 slo=0.
	 rizero=0.
	 ri=0.
	 rik=0.
	 ro=0.
!	 wf=0.
	 rikzero=0.
	 temp=0.
	 ir=0.
	 depth=0.
	 zmax=0.
! Choose file extension for grid files, Added 4/14/2010
	 grxt='.txt'
	 if(lasc) grxt='.asc'
! *****************************************************************
!  read gridded data from GIS
	write (*,*) 'Reading input grids'
     	write(u(19),*) 'Input file name,            Cell count'
!  read slope angles
	call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
     &	slo,pf1,sctr,imax,temp,u(1),slofil,param,header,u(19))
     	  write(u(19),*) 'Slope angle grid'
    	  write(u(19),*) trim(slofil),sctr,' data cells'
	  if (imax>sctr) imx1=sctr
	  write(u(19),*) 'Set number of cells for pore-pressure&
	  & factor of safety computations to ',imx1
	  if(imx1<imax) then
	    write(*,*) 'Check initialization file: '
	    write(*,*) 'Value of imax exceeds number of grid cells.'
	    write(u(19),*) 'Check initialization file: '
	    write(u(19),*) 'Value of imax > number of grid cells.'
	  end if
	  slo=slo*dg2rad ! convert slope angles to radians
!  read property zone numbers, zo
	if(nzon==1) then
	  zo=1 ! if only one zone, all values of zone grid equal 1.
	  write(*,*) 'One property zone, no grid required!'
	  write(u(19),*) 'One property zone, no grid required!'
	  parami=param ! added 7/29/2008 RLB
	else
	  call irdgrd(grd,col,ncol,nrow,celsiz,nodata,mnd,&
          &zo,pf2,sctr,imax,temp,u(15),zonfil,parami,header,u(19))
    	  write(u(19),*) 'Property zone grid'
      	  write(u(19),*) trim(zonfil),sctr,' data cells'
     	  if(sctr/=imx1) then
     	    write (*,*) 'Grid mismatch ',trim(zonfil)
     	    write (*,*) 'Correct property-zone grid and/or initializtion file.' 
     	    write (u(19),*) 'Grid mismatch ',trim(zonfil)
     	    write (u(19),*) 'Correct property-zone grid and/or initializtion file.' 
     	    close(u(19))
     	    pause 'Press return/enter to quit'
     	    stop '-1'
     	  end if
     	  maxzo=maxval(zo)
     	  if (maxzo/=nzon) then
     	    write (*,*) 'Maximum zone number does not equal number of property zones!'
     	    write (*,*) 'Correct property-zone grid and/or initializtion file.' 
     	    write (u(19),*) 'Maximum zone number does not equal number of property zones!'
     	    write (u(19),*) 'Correct property-zone grid and/or initializtion file.' 
     	    close(u(19))
     	    pause 'Press return/enter to quit'
     	    stop '-1'
     	  end if
	end if
!
!
! SR__
!seed1 = values (7) * values (8)
!seed2 = seed1 - seed1/2
  seed1=123
  seed2=456

igen  = 1

write (*,*) ' *** [ SR ] : starting random generators ... '
write (*,* ) ' ( seed1 , seed2 ) == ( ' , seed1, ' , ' , seed2, ' )'


 SELECT CASE (distribution)
 	CASE ('gaussian')
		write (*,*) ' *** [ SR ] : calling gaussian subroutine '
 		CALL generate_gaussian_distributions ( imax , igen , seed1 , seed2 )
 	CASE ('uniform')
		write (*,*) ' *** [ SR ] : calling uniform subroutine '
		CALL generate_uniform_distributions  ( imax , igen , seed1 , seed2 )
 	CASE DEFAULT
		write (*,*) ' *** [ SR ] : calling nothing (Deterministic case limit) '
 END SELECT
	 
 write (*,*) ' *** [ SR ] : Ending random generators ...'
! __SR
!
!
! *********************     	
!  read background infiltration rate, Isub0 
	if (crizero.lt.0) then 
	  call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
          &rizero,pf1,sctr,imax,temp,u(16),rizerofil,param,header,u(19))
    	  write(u(19),*) 'Background infiltration rate grid'
     	  write(u(19),*) trim(rizerofil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(rizerofil)
  	else
	  rizero=crizero
  	end if
!  read initial depth to water table, 
	if (dep.lt.0) then  
	  call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
          &depth,pf1,sctr,imax,temp,u(10),depfil,param,header,u(19))
    	  write(u(19),*) 'Initial water-table depth grid'
     	  write(u(19),*) trim(depfil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(depfil)
  	else
	  depth=dep
  	end if
!  read depth to base of potential slide, zmax
	if (czmax.lt.0) then 
	call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
     &	zmax,pf1,sctr,imax,temp,u(11),zfil,param,header,u(19))
    	write(u(19),*) 'Maximum depth grid'
     	write(u(19),*) trim(zfil),sctr,' data cells'
     	if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(zfil)
  	else
	zmax=czmax
  	end if
!  read digital elevations, elev !! added 2/24/ 2010, unit number changed to 12 12/6/2010
	if (flag<= -4) then 
	  call srdgrd(grd,col,ncol,nrow,celsiz,nodat,&
     &	  elev,pf1,sctr,imax,temp,u(12),elevfil,param,header,u(19))
    	  write(u(19),*) 'Elevation grid'
     	  write(u(19),*) trim(elevfil),sctr,' data cells'
     	  if(sctr/=imx1) write (u(19),*) 'Grid mismatch ',trim(zfil)
  	end if
! *****************************************************************
     	write(u(19),*) '---------------******---------------'
! test and adjust (if necessary) steady background infiltration rates
! SR__ 
        if (ks_sd_glob_flag) then    
           call steady_pro(sumex,u(19),imx1)
        else
           call steady_det(sumex,u(19),imx1)
        end if
! __SR
!
! conduct runoff routing and adjust transient infiltration rates
! SR__
        if (ks_sd_glob_flag) then     
           call rnoff_pro(grd,sumex,imx1,celsiz,param,parami,nodat,&
                & nodata,mnd,sctr,ncol,nrow,header,test1,u,ulen)
        else
           call rnoff_det(grd,sumex,imx1,celsiz,param,parami,nodat,&
                & nodata,mnd,sctr,ncol,nrow,header,test1,u,ulen)
        end if
! __SR
! Deallocate arrays that are no longer needed
	deallocate (ri,ro,wf)
	deallocate (pf2,indx,nxt,dsctr,dsc)
     	write(u(19),*) '---------------******---------------'
     	write(u(19),*) 'Input file name,          Cell count'
! *****************************************************************
! compute pore pressure distributions for either fully saturated or
! partially saturated conditions.
! Partially saturated zone overlies saturated zone
! Allocate and initialize new arrays
	allocate (fsmin(imax*nout),pmin(imax*nout),zfmin(imax*nout))
	allocate (p(nzs+1),ptran(nzs+1),pzero(nzs+1),bline(nzs+1))
	allocate (fc(nzs+1),fw(nzs+1),thz(nzs+1),kz(nzs+1),trz(nzs+1))
	allocate (nvu(imax),nv(imax),uwsp(nzs+1),gs(nzon))
	allocate (chi(nzs+1))
	if (flag<= -4) then ! moved 5/3/2010, flag =-4,-5,-6 produces ijz or xyz output 12/22/2010 RLB
	  allocate(ix(imax),jy(imax))
	  ix=0;jy=0
  	end if
	fsmin=0.
	zfmin=0.
	pmin=0.
	p=0.
	ptran=0.
	pzero=0.
	bline=0.
	fc=0.
	fw=0.
	nv=0
	nvu=0
! determine number of time steps needed	  
	  kper=nper
	  if (t>capt(nper+1)) then
	    kper=nper+1 
	  else
    	    do k=1,nper ! find the period that contains t
    	      if(t>=capt(k) .and. t<=capt(k+1)) kper=k
            end do
	  end if 
	  if (tx<1) tx=1
	  nts=kper*tx ! number of time-steps from 0 to t
 	  tns=float(nts)
 	  tmin=0.
 	  tmax=t 
	  tinc=(tmax-tmin)/tns
! compute output pointers	
	  allocate(jsav(nts+1))  
	  jsav=0
	  write (u(19),*) '******** Output times ********'
	  write (u(19),*) 'number, timestep #,  time'
	  write (*,*) '******** Output times ********'
	  write (*,*) 'number, timestep #,  time'
	  lwarn=.false.
	  do k=1,nout
	    ts=tmin
	    do j=1,nts
	      if(tsav(k)>=ts .and. tsav(k)<ts+tinc) then
	        if(tsav(k)/=ts) lwarn=.true.
	        jsav(j)=k
	        ksav(k)=j
	        tsav(k)=ts
	        exit
	      else if(tsav(k)>=tmax) then
	        jsav(nts+1)=k
	        ksav(k)=nts+1
	        tsav(k)=tmax
	        exit
	      end if
	      ts=ts+tinc
	    end do
	    if(lwarn) then
	    write(u(19),*) 'One or more specified output times unavailable, '
	    write(u(19),*) 'Nearest available time substitued.'
	    write(*,*) 'One or more specified output times unavailable, '
	    write(*,*) 'Nearest available time substitued.'
	    end if 
	    write(u(19),*) k,ksav(k),tsav(k)
	    write(*,*) k,ksav(k),tsav(k)
	  end do
! allocate and initialize additonal model arrays	  
	  allocate (r(nmax),q(kper),qtime(2*nts+1),qb(nts),tcap(nts+2))
	  if(outp(7)) then
	    allocate(rik1(imax*(nts+1))) ! corrected 6/30/2008, RLB
	    rik1=0.
	  end if
	  eps=1.0e-18
	  nmax2=0
	  nmin=1+nmax
	  ncc=0;nccs=0
	  tis=tiny(x1)
	  write(*,*) 'Starting computations of pressure head &
          &and factor of safety'
! prepare file, header, and arrays for generating list files, Added 4/21/2010, revised 5/3/2010, 12/6/2010
          if(flag< 0) then
	     umax=maxval(u) ! added 12/6/2010, RLB
	     uijz(1)=umax+1
	     call prpijz(u(2),u(19),profil,ncol,nrow,header,vrsn) !Revised 12/23/2010
	   end if
! ---------------------------------------------------------------
	if(unsat0) then
 	  smt=0.05d0; lard=12.d0 ! test values for early-time
    
          do j=1,nzon 
             gs(j)=((uws(j)/uww)-ths(j))/(1-ths(j))
          end do


	  if(mmax.lt.0) then ! infinite depth model
	    mmax=20
	    write(*,*) 'Calling unsaturated infinite-depth model'
     !  4/14/2010 RLB added logging of main subroutines handling infiltration 	    
	    write(u(19),*) 'Unsaturated infinite-depth model, unsinf()'
! SR__            
            if (ks_sd_glob_flag) then 
               call unsinf_pro(imx1,u(19),u(2),ncc,nccs,profil)
            else
               call unsinf_det(imx1,u(19),u(2),ncc,nccs,profil)
            end if
! __SR
     	  else ! finite depth model
	    write(*,*) 'Calling unsaturated finite-depth model'
           write(u(19),*) 'Unsaturated finite-depth model, unsfin()'
! SR__
            if (ks_sd_glob_flag) then
               call unsfin_pro(imx1,u(19),u(2),ncc,nccs,profil)
            else
               call unsfin_det(imx1,u(19),u(2),ncc,nccs,profil)
            end if
! __SR
	  end if
	else ! Saturated zone extends to ground surface
	  write(*,*) 'Ignoring unsaturated zone'
	  if(tx==1 .and. nout==1) then
	    deallocate (rizero,ks,ir)
!  compute pore-pressure distributions and factor of safety
	    outfil=trim(folder)//trim(profil)//trim(suffix)//'.txt'
	    nccs=0
	    if(mmax.lt.0) then ! infinite depth model
	      mmax=20
	      write(*,*) 'Calling saturated infinite-depth model'
	      write(u(19),*) 'Saturated infinite-depth model, iverson()'
	      call iverson(imx1,u(2),outfil,u(19))
     	    else ! finite depth model
	      write(*,*) 'Calling saturated finite-depth model'
	      write(u(19),*) 'Saturated finite-depth model, savage()'
	      call savage(imx1,u(2),outfil,u(19),nccs)
	    end if
	  else
	    if(mmax.gt.0) then	
	      write(*,*) 'Calling multistep saturated finite-depth model'
              write(u(19),*) 'Multistep saturated finite-depth model, satfin()'
! SR__
              if (ks_sd_glob_flag) then
                 call satfin_pro(imx1,u(19),u(2),nccs,profil)
              else
                 call satfin_det(imx1,u(19),u(2),nccs,profil)
              end if
! __SR
	    else
	      write(*,*) 'Calling multistep saturated infinite-depth model'
              write(u(19),*) 'Multistep saturated infinite-depth model, satinf()'
! SR__
              if (ks_sd_glob_flag) then
                 call satinf_pro(imx1,u(19),u(2),nccs,profil)
              else
                 call satinf_det(imx1,u(19),u(2),nccs,profil)
              end if
! __SR
	    end if
	  end if
	end if
! *****************************************************************
!  write output grid files (File extension changed from ".asc" to ".txt", 7/29/2008, RLB)
! 4/14/2020 added option to let file extension be either ".asc" or ".txt"
  101	continue
	write(*,*) 'Saving results'
	ti=tiny(param(m))
	do j=1,nout
	  write(stp,'(i4)') j
	  stp=adjustl(stp)
	  if (outp(3)) then ! minimum factor of safety
	    tfg=0.
	    do i=1,imx1
	      tfg(i)=fsmin(i+(j-1)*imax)

! to be removed (tests for z5 zone-zoom)

              if (zo(i).eq.5.AND.j.eq.nout) then
                 if (tfg(i).ge.1.0.AND.tfg(i).le.1.1) then 
                    totfsmin_z5=totfsmin_z5+1
                 end if 
                 totcells_z5= totcells_z5 +1 
!                 if (abs(slo(zo(i)))>1.e-5) then
!                    if (phi_sd_glob_flag) then
!                       z5fs_1(i)=tan(phi_norm(i))/tan(slo(zo(i)))
!                    else
!                       z5fs_1(i)=tan(phi(zo(i)))/tan(slo(zo(i)))
!                    end if
!                 else
!                    z5fs_1(i)=10
!                 end if
!                 z5fs_2(i)=tfg(i) - z5fs_1 (i)
!              else
!                 z5fs_1(i)=-9999
!                 z5fs_2(i)=-9999
              end if
! tmp__SR

           end do

           

! SR__tmp
!     open (UNIT=8,file=trim(fminfilz5_1),status='unknown',err=410)
!     open (UNIT=9,file=trim(fminfilz5_2),status='unknown',err=410)

!            outfil=trim(folder)//trim(fminfilz5_1)//trim(suffix)//'_'//trim(stp)//grxt
!            outfil=trim(folder)//'fsz5_1.txt'
!   	    call ssvgrd(z5fs_1,imax,pf1,nrow,ncol,u(22),test1,param,u(19),outfil,ti,header)


 !           outfil=trim(folder)//trim(fminfilz5_2)//trim(suffix)//'_'//trim(stp)//grxt
!            outfil=trim(folder)//'fsz5_2.txt'
 !  	    call ssvgrd(z5fs_2,imax,pf1,nrow,ncol,u(23),test1,param,u(19),outfil,ti,header)

! tmp__SR


	    outfil=trim(folder)//trim(fminfil)//trim(suffix)//'_'//trim(stp)//grxt
   	    call ssvgrd(tfg,imax,pf1,nrow,ncol,u(4),test1,param,u(19),&
            &outfil,ti,header)
	  end if
          ratio=real(totfsmin_z5)/real(totcells_z5)
          write (9,"(I5, I5, F10.3)") totfsmin_z5 , totcells_z5 , ratio
!          write (9,"(F10.3)") single_fs 


	  if (outp(4)) then ! depth of minimum factor of safety
	    tfg=0.
	    do i=1,imx1
	      tfg(i)=zfmin(i+(j-1)*imax)
	    end do
	    outfil=trim(folder)//trim(zfminfil)//trim(suffix)//'_'//trim(stp)//grxt
   	    call ssvgrd(tfg,imax,pf1,nrow,ncol,u(5),test1,param,u(19),&
            &outfil,ti,header)
	  end if
 	  if (outp(5)) then ! pressure head at depth of minimum factor of safety
	    tfg=0.
	    do i=1,imx1
	      tfg(i)=pmin(i+(j-1)*imax)
	    end do
 	    outfil=trim(folder)//trim(pminfil)//trim(suffix)//'_'//trim(stp)//grxt
   	    call ssvgrd(tfg,imax,pf1,nrow,ncol,u(6),test1,param,u(19),&
            &outfil,ti,header)
	  end if
        end do
	if (outp(7) .and. unsat0) then ! incremental basal flux, unsaturated zone
	    do j=1,nts
	      ir=0.
	      do i=1,imx1
	        ir(i)=rik1(i+(j-1)*imax)
	      end do
	      ti=tiny(param(m))
	      write(scratch,'(i6)') j
	      scratch=adjustl(scratch)
	      irfil='TRunszfluxTS'//trim(scratch)//trim(suffix)//grxt
	      outfil=trim(folder)//trim(irfil)
! to remove
   	      call ssvgrd(ir,imax,pf1,nrow,ncol,u(24),test1,&
     &	      param,u(19),outfil,ti,header)
!          call ssvgrd(ir,imax,pf1,nrow,ncol,u(9),test1,&
!     &	      param,u(19),outfil,ti,header)
	    end do
	  end if	  
 	if (ncc>0) then ! non-convergent cells, unsaturated zone (12/6/2010, changed unit # from 7 to 14)
 	  outfil=trim(folder)//ncvfil//'UZ_'//trim(suffix)//grxt
   	  call isvgrd(nvu,imax,pf1,nrow,ncol,u(14),test,test,mnd,&
          & parami,u(19),outfil,ti,header)
	end if
 	if (nccs>0) then ! non-convergent cells, saturated zone
 	  outfil=trim(folder)//ncvfil//'SZ_'//trim(suffix)//grxt
   	  call isvgrd(nv,imax,pf1,nrow,ncol,u(14),test,test,mnd,&
          & parami,u(19),outfil,ti,header)
	end if

   	write (*,*) 'TRIGRS finished!'
     	write (u(19),*) 'TRIGRS finished normally'
     	call date_and_time(date,time)
     	write (u(19),*) 'Date ',date(5:6),'/',date(7:8),'/',date(1:4)
	write (u(19),*) 'Time ',time(1:2),':',time(3:4),':',time(5:6)
	close (u(19))


! SR__ 
write (*, * ) ' *** [ SR ] : START : deallocation ... '
   if (allocated(rifil )) deallocate (rifil)
   if (allocated(ths   )) deallocate (ths)
   if (allocated(thr   )) deallocate (thr)
   if (allocated(alp   )) deallocate (alp)
   if (allocated(unsat )) deallocate (unsat)
   if (allocated(fsmin )) deallocate (fsmin)
   if (allocated(pmin  )) deallocate (pmin)
   if (allocated(zfmin )) deallocate (zfmin)
   if (allocated(q     )) deallocate (q)
!   if (allocated(qb    )) deallocate (qb)
!   if (allocated(tcap  )) deallocate (tcap)
   if (allocated(qtime )) deallocate (qtime)
   if (allocated(bline )) deallocate (bline)
   if (allocated(fc    )) deallocate (fc)
   if (allocated(fw    )) deallocate (fw)
   if (allocated(thz   )) deallocate (thz)
   if (allocated(kz    )) deallocate (kz)
   if (allocated(trz   )) deallocate (trz)
   if (allocated(nvu   )) deallocate (nvu)
   if (allocated(nv    )) deallocate (nv)
   if (allocated(uwsp  )) deallocate (uwsp)
   if (allocated(gs    )) deallocate (gs)
   if (allocated(jsav  )) deallocate (jsav)
   if (allocated(tsav  )) deallocate (tsav)
   if (allocated(ksav  )) deallocate (ksav)
   if (allocated(slo   )) deallocate (slo)
   if (allocated(pf1   )) deallocate (pf1)
   if (allocated(rik   )) deallocate (rik)
   if (allocated(zmax  )) deallocate (zmax)
   if (allocated(zo    )) deallocate (zo)
   if (allocated(tfg   )) deallocate (tfg)
   if (allocated(pzero )) deallocate (pzero) 
   if (allocated(c     )) deallocate (c)
   if (allocated(phi   )) deallocate (phi)
   if (allocated(uws   )) deallocate (uws)
   if (allocated(dif   )) deallocate (dif)
   if (allocated(cri   )) deallocate (cri)
   if (allocated(capt  )) deallocate (capt)
   if (allocated(rikzero)) deallocate (rikzero)
   if (allocated(depth  )) deallocate (depth)
   
   if (allocated(c_norm  )) deallocate (c_norm)
   if (allocated(phi_norm)) deallocate (phi_norm)
   if (allocated(uws_norm)) deallocate (uws_norm)   
   if (allocated(dif_norm)) deallocate (dif_norm)
   if (allocated(ks_norm )) deallocate (ks_norm)
   if (allocated(ths_norm)) deallocate (ths_norm)
   if (allocated(thr_norm)) deallocate (thr_norm)
   if (allocated(alp_norm)) deallocate (alp_norm)      

   if (allocated(c_sd  )) deallocate (c_sd)
   if (allocated(phi_sd)) deallocate (phi_sd)
   if (allocated(uws_sd)) deallocate (uws_sd)   
   if (allocated(dif_sd)) deallocate (dif_sd)
   if (allocated(ks_sd )) deallocate (ks_sd)
   if (allocated(ths_sd)) deallocate (ths_sd)
   if (allocated(thr_sd)) deallocate (thr_sd)
   if (allocated(alp_sd)) deallocate (alp_sd)      

   if (allocated(c_min  )) deallocate (c_min  )
   if (allocated(phi_min)) deallocate (phi_min)
   if (allocated(uws_min)) deallocate (uws_min)   
   if (allocated(dif_min)) deallocate (dif_min)
   if (allocated(ks_min )) deallocate (ks_min )
   if (allocated(ths_min)) deallocate (ths_min)
   if (allocated(thr_min)) deallocate (thr_min)
   if (allocated(alp_min)) deallocate (alp_min)      

   if (allocated(c_max  )) deallocate (c_max  )
   if (allocated(phi_max)) deallocate (phi_max)
   if (allocated(uws_max)) deallocate (uws_max)   
   if (allocated(dif_max)) deallocate (dif_max)
   if (allocated(ks_max )) deallocate (ks_max )
   if (allocated(ths_max)) deallocate (ths_max)
   if (allocated(thr_max)) deallocate (thr_max)
   if (allocated(alp_max)) deallocate (alp_max)      

   if (allocated(temp   )) deallocate (temp   )
   if (allocated(p      )) deallocate (p      )
   if (allocated(ptran  )) deallocate (ptran  )
   if (allocated(r      )) deallocate (r      )

   write (*, * ) ' *** [ SR ] : END : deallocation.  '
! __SR 

	stop '0'
! Error reporting 	
  410	continue
  	write (*,*) 'Error opening output file'
	write (*,*) '--> ',outfil
	write (*,*) 'Check file path and status'
  	write (u(19),*) 'Error opening output file'
	write (u(19),*) '--> ',outfil
	write (u(19),*) 'Check file path and status'
	pause 'Press RETURN to exit'
	stop '410'
   	end program trigrs

